<script setup lang="ts">
import { computed, ref } from 'vue';

import { useVbenDrawer, useVbenModal } from '@vben/common-ui';
import { $t } from '@vben/locales';
import { cloneDeep } from '@vben/utils';

import { InputSearch } from 'ant-design-vue';

import { useVbenForm } from '#/adapter/form';
import { addMaintOrditm, getMaintOrditm, updateMaintOrditm } from '#/api/eims/maint-orditm';
import { listMaintStand } from '#/api/eims/maint-stand';
import orderModal from '#/views/eims/components/miant-order-modal.vue';
import userModal from '#/views/eims/components/user-modal.vue';

import { drawerSchema } from './data';

const emit = defineEmits<{ reload: [] }>();

const isUpdate = ref(false);
const title = computed(() => {
  return isUpdate.value ? $t('pages.common.edit') : $t('pages.common.add');
});

const [BasicForm, formApi] = useVbenForm({
  commonConfig: {
    formItemClass: 'col-span-2',
    componentProps: {
      class: 'w-full'
    },
    labelWidth: 120
  },
  schema: drawerSchema(),
  showDefaultActions: false,
  wrapperClass: 'grid-cols-2'
});

const [BasicDrawer, drawerApi] = useVbenDrawer({
  onCancel: handleCancel,
  onConfirm: handleConfirm,
  async onOpenChange(isOpen) {
    if (!isOpen) {
      return null;
    }
    drawerApi.drawerLoading(true);
    const { id } = drawerApi.getData() as { id?: number | string };
    isUpdate.value = !!id;
    // 更新 && 赋值
    if (isUpdate.value && id) {
      const record = await getMaintOrditm(id);
      await formApi.setValues(record);
    }

    drawerApi.drawerLoading(false);
  }
});

/**
 * 工单明细的加载
 */
async function setupOrditmOptions(equId: any) {
  const params = { equId };
  const userPageResult = await listMaintStand({
    pageNum: 1,
    pageSize: 50,
    ...params
  });
  const options = userPageResult?.rows.map((item) => ({
    label: item.name,
    value: item.name
  }));
  // 筛选
  const filterOption = (input: string, option: any) => {
    return option.label.toLowerCase().includes(input.toLowerCase());
  };

  const placeholder = options.length > 0 ? '请选择' : '该工单暂无明细';
  formApi.updateSchema([
    {
      componentProps: {
        options,
        placeholder,
        filterOption,
        async onSelect(name: string) {
          const item = userPageResult?.rows.find((i) => i.name === name);
          await formApi.setValues({ itmAction: item?.stand });
        }
      },
      fieldName: 'itmName'
    }
  ]);
}

async function handleConfirm() {
  try {
    drawerApi.drawerLoading(true);
    const { valid } = await formApi.validate();
    if (!valid) {
      return;
    }
    const data = cloneDeep(await formApi.getValues());
    await (isUpdate.value ? updateMaintOrditm(data) : addMaintOrditm(data));
    emit('reload');
    await handleCancel();
  } catch (error) {
    console.error(error);
  } finally {
    drawerApi.drawerLoading(false);
  }
}

async function handleCancel() {
  drawerApi.close();
  await formApi.resetForm();
}

// order modal
const [OrderModal, orderModalApi] = useVbenModal({
  connectedComponent: orderModal,
  draggable: true,
  title: '选择工单'
});

function handleOrderModal() {
  orderModalApi.setData({});
  orderModalApi.open();
}

/**
 * 更新选择的工单
 * @param order
 */
async function updateOrderSelect(order: any) {
  await formApi.setValues({ 'orderId': order.id, 'orderName': order.maintCode, 'equId': order.equId, 'equName': order.equName });
  await setupOrditmOptions(order?.equId);
}

// user modal
const [UserModal, userModalApi] = useVbenModal({
  connectedComponent: userModal,
  draggable: true,
  title: '选择成员'
});

/**
 * 打开选择用户
 */
function handleUserModal() {
  userModalApi.setData({});
  userModalApi.open();
}

/**
 * 更新选择的用户
 * @param user
 */
async function updateUserSelect(user: any) {
  await formApi.setValues({ 'itmUser': user.userId, 'itmUserName': user.nickName });
}
</script>

<template>
  <BasicDrawer :close-on-click-modal="false" :title="title" class="w-[600px]">
    <BasicForm>
      <template #orderName="slotProps">
        <InputSearch :enter-button="true" placeholder="请选择" @search="handleOrderModal" v-bind="slotProps" />
      </template>
      <template #itmUserName="slotProps">
        <InputSearch :enter-button="true" placeholder="请选择" @search="handleUserModal" v-bind="slotProps" />
      </template>
    </BasicForm>
    <OrderModal class="w-[1200px]" @update-select="updateOrderSelect" />
    <UserModal class="w-[1200px]" @select-user="updateUserSelect" />
  </BasicDrawer>
</template>
